/*
 * Copyright (c) 2007, Ryo Ichinose
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright 
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the auther nor the 
 *      names of its contributors may be used to endorse or promote products 
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/sleep.h>

#include "timer0.h"
#include "util.h"


#define OCR0_AS_TOP 99

static void timer0_comp_interrupt(void);


int
main(void)
{
	DDRD |= _BV(DDD5);  /* PD5 ԥϤ */
	pullup();

	timer0_comp_enable();
	timer0_comp_set_interrupt(timer0_comp_interrupt);
	timer0_set_wave_mode(WGM0_CTC);
	timer0_set_count(0);
	timer0_set_ocr0(OCR0_AS_TOP);

	sei();

	/* (8MHz/8)/100 = 10KHz */
	timer0_start(TIMER0_CK_8);
	
	while (1){
		sleep_mode();
	}

	return 0; /* ٹ򤱤뤿 */
}


static void 
timer0_comp_interrupt(void)
{
	static int duty[3][2] = {
		{21, 179},
		{15, 185},
		{ 9, 191}
	};

	static int dir = 0;   /*  */
	static int state = 0; /* H/L  */
	static int count = 0;
	static int pulse_count = 0;

	/* 1Ѥ */
	if (++ count == 10000) {
		dir = (dir + 1) % 3;
		state = count = pulse_count = 0;
		PORTD |= _BV(PORTD5);
		return;
	}

	/* PWM  */
	if (++ pulse_count == duty[dir][state]) {
		state = (state + 1) % 2;
		pulse_count = 0;
		PORTD ^= _BV(PORTD5);
	}
}

/* End of main.c */
