/*
 * Copyright (c) 2007, Ryo Ichinose
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright 
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright 
 *      notice, this list of conditions and the following disclaimer in the 
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of the auther nor the 
 *      names of its contributors may be used to endorse or promote products 
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ___TIMER0_H_INCLUDED_
#define ___TIMER0_H_INCLUDED_

#include <avr/io.h>
#include <inttypes.h>


/* ޡåѥޥ */
#define TIMER0_STOP     0x00
#define TIMER0_CK       (_BV(CS00))
#define TIMER0_CK_8     (_BV(CS01))
#define TIMER0_CK_64    (_BV(CS01) | _BV(CS00))
#define TIMER0_CK_256   (_BV(CS02))
#define TIMER0_CK_1024  (_BV(CS02) | _BV(CS00))

/* ӽϥ⡼ѥޥ */
#define COM0_NONE       0x00
#define COM0_TOGGLE     (_BV(COM00))
#define COM0_HIGH_FIRST (_BV(COM01))
#define COM0_LOW_FIRST  (_BV(COM01) | _BV(COM00))

/* ȷѥޥ */
#define WGM0_NORMAL     0x00
#define WGM0_PWM        (_BV(WGM00))
#define WGM0_CTC        (_BV(WGM01))
#define WGM0_FAST_PWM   (_BV(WGM01) | _BV(WGM00))

#define timer0_ov_enable()       { TIMSK |= _BV(TOIE0); }
#define timer0_ov_disable()      { TIMSK &= ~_BV(TOIE0); }
#define timer0_comp_enable()     { TIMSK |= _BV(OCIE0); }
#define timer0_comp_disable()    { TIMSK &= ~_BV(OCIE0); }
#define timer0_clear_com_mode()  { TCCR0 &= ~(_BV(COM01) | _BV(COM00)); }
#define timer0_clear_wave_mode() { TCCR0 &= ~(_BV(WGM01) | _BV(WGM00)); }

#define timer0_stop()\
	{ TCCR0 &= ~(_BV(CS02) | _BV(CS01) | _BV(CS00)); }

#define timer0_set_com_mode(mode)  {\
	timer0_clear_com_mode();\
	TCCR0 |= (mode) & (_BV(COM01) | _BV(COM00));\
}

#define timer0_set_wave_mode(mode) {\
	timer0_clear_wave_mode();\
	TCCR0 |= (mode) & (_BV(WGM01) | _BV(WGM00));\
}

#define timer0_start(clock) {\
	timer0_stop();\
	TCCR0 |= (clock) & (_BV(CS02) | _BV(CS01) | _BV(CS00));\
}

/*
 * TCNT0 ꤹ륫ͤꤹ롣
 */
static inline void
timer0_set_count(uint8_t count)
{
	TCNT0 = count;
}


/*
 * OCR0 ꤹ륫ͤꤹ롣
 */
static inline void
timer0_set_ocr0(uint8_t ocr0)
{
	OCR0 = ocr0;
}


void timer0_ov_set_interrupt(void (*func)(void));
void timer0_comp_set_interrupt(void (*func)(void));


#endif

/* End of timer0.h */
